@@ -3,9 +3,8 @@ use crate::{Scalar,Point,Value,Coord,Error,EyrosErrorKind,Overlap,RA,Root,
   query::QStream, tree_file::TreeFile, SetupFields};
 use async_std::{sync::{Arc,Mutex}};
 use crate::unfold::unfold;
-use std::collections::{HashMap,HashSet};
-#[path="./join.rs"] mod join;
-use join::Join;
+use std::collections::{HashMap,HashSet,VecDeque};
+use futures::future::join_all;
 
 pub type TreeId = u64;
 #[derive(Debug,Clone,PartialEq)]
@@ -99,15 +98,41 @@ macro_rules! impl_tree {
           count: build.count + (build.range.1-build.range.0) - (range.1-range.0),
         }
       }
-      fn build(&mut self, build: &Build) -> $Node<$($T),+,V> {
+      fn build(&mut self, build: &Build, is_rm: bool) -> $Node<$($T),+,V> {
         let rlen = build.range.1 - build.range.0;
+        let mut build_ext = false;
         if rlen == 0 {
           return $Node::Data(vec![],vec![]);
         } else if rlen < self.fields.inline || rlen <= 2 {
           let inserts = &self.inserts;
-          return $build_data(&self.sorted[build.range.0..build.range.1].iter().map(|i| {
+          let data = $build_data(&self.sorted[build.range.0..build.range.1].iter().map(|i| {
             inserts[*i].clone()
           }).collect::<Vec<(($(Coord<$T>),+),InsertValue<'_,($(Coord<$T>),+),V>)>>());
+          if data.count_bytes() >= self.fields.inline_max_bytes {
+            build_ext = true;
+          } else {
+            return data;
+          }
+        } else if !is_rm && rlen <= self.fields.ext_records {
+          build_ext = true;
+        }
+        if build_ext {
+          let r = self.next_tree;
+          let tr = TreeRef {
+            id: r,
+            bounds: $get_bounds(
+              self.sorted[build.range.0..build.range.1].iter().map(|i| *i),
+              self.inserts
+            ),
+          };
+          self.next_tree += 1;
+          let inserts = &self.inserts;
+          let root = $build_data(&self.sorted[build.range.0..build.range.1].iter().map(|i| {
+            inserts[*i].clone()
+          }).collect::<Vec<(($(Coord<$T>),+),InsertValue<'_,($(Coord<$T>),+),V>)>>());
+          let t = $Tree::new(Arc::new(root));
+          self.ext_trees.insert(r, Arc::new(Mutex::new(t)));
+          return $Node::Data(vec![],vec![tr]);
         }
         if build.level >= self.fields.max_depth || build.count >= self.fields.max_records {
           let r = self.next_tree;
@@ -119,7 +144,7 @@ macro_rules! impl_tree {
             ),
           };
           self.next_tree += 1;
-          let t = $Tree::new(Arc::new(self.build(&build.ext())));
+          let t = $Tree::new(Arc::new(self.build(&build.ext(), is_rm)));
           self.ext_trees.insert(r, Arc::new(Mutex::new(t)));
           return $Node::Data(vec![],vec![tr]);
         }
@@ -207,7 +232,7 @@ macro_rules! impl_tree {
                   hset.contains(j)
                 })
               );
-              (*bitfield, Arc::new(self.build(&next)))
+              (*bitfield, Arc::new(self.build(&next, is_rm)))
             }).collect()
           }),+,
           _ => panic!["unexpected level modulo dimension"]
@@ -230,7 +255,7 @@ macro_rules! impl_tree {
                   }
                 })
               );
-              Arc::new(self.build(&next))
+              Arc::new(self.build(&next, is_rm))
             });
             let ranges = pv.iter().zip(pv.iter().skip(1));
             for range in ranges {
@@ -242,7 +267,7 @@ macro_rules! impl_tree {
                   intersect_coord(&(inserts[*j].0).$i, range.0, range.1)
                 })
               );
-              nodes.push(Arc::new(self.build(&next)));
+              nodes.push(Arc::new(self.build(&next, is_rm)));
             }
             if (pv.len() > 1) {
               nodes.push({
@@ -258,7 +283,7 @@ macro_rules! impl_tree {
                     }
                   })
                 );
-                Arc::new(self.build(&next))
+                Arc::new(self.build(&next, is_rm))
               });
             }
             nodes
@@ -286,10 +311,12 @@ macro_rules! impl_tree {
           nodes,
         }
       }
-      pub fn build<'a>(fields: Arc<SetupFields>,
-      inserts: &[(($(Coord<$T>),+),InsertValue<'a,($(Coord<$T>),+),V>)],
-      next_tree: &mut TreeId)
-      -> (Option<TreeRef<($(Coord<$T>),+)>>,CreateTrees<$Tree<$($T),+,V>>) {
+      pub fn build<'a>(
+        fields: Arc<SetupFields>,
+        inserts: &[(($(Coord<$T>),+),InsertValue<'a,($(Coord<$T>),+),V>)],
+        next_tree: &mut TreeId,
+        is_rm: bool,
+      ) -> (Option<TreeRef<($(Coord<$T>),+)>>,CreateTrees<$Tree<$($T),+,V>>) {
         if inserts.is_empty() { return (None, HashMap::new()) }
         let mut mstate = $MState {
           fields,
@@ -317,7 +344,7 @@ macro_rules! impl_tree {
           range: (0, inserts.len()),
           level: 0,
           count: 0,
-        });
+        }, is_rm);
         *next_tree = mstate.next_tree;
         let tr = TreeRef {
           id: *next_tree,
@@ -349,35 +376,134 @@ macro_rules! impl_tree {
       fn empty() -> Self {
         Self { root: Arc::new($Node::Data(vec![],vec![])) }
       }
-      fn build<'a>(fields: Arc<SetupFields>,
-      rows: &[(($(Coord<$T>),+),InsertValue<'a,($(Coord<$T>),+),V>)], next_tree: &mut TreeId)
-      -> (Option<TreeRef<($(Coord<$T>),+)>>,HashMap<TreeId,Arc<Mutex<Self>>>) {
-        $Branch::build(fields, rows, next_tree)
+      fn build<'a>(
+        fields: Arc<SetupFields>,
+        rows: &[(($(Coord<$T>),+),InsertValue<'a,($(Coord<$T>),+),V>)],
+        next_tree: &mut TreeId,
+        is_rm: bool,
+      ) -> (Option<TreeRef<($(Coord<$T>),+)>>,HashMap<TreeId,Arc<Mutex<Self>>>) {
+        $Branch::build(fields, rows, next_tree, is_rm)
       }
       fn list(&mut self) -> (Vec<(($(Coord<$T>),+),V)>,Vec<TreeRef<($(Coord<$T>),+)>>) {
-        let mut cursors = vec![Arc::clone(&self.root)];
+        let mut cursors = VecDeque::new();
+        cursors.push_back(self.root.clone());
         let mut rows = vec![];
         let mut refs = vec![];
-        while let Some(c) = cursors.pop() {
+        while let Some(c) = cursors.pop_front() {
           match c.as_ref() {
             $Node::Branch(branch) => {
               for (_bitfield,b) in branch.intersections.iter() {
-                cursors.push(Arc::clone(b));
+                cursors.push_back(b.clone());
               }
               for b in branch.nodes.iter() {
-                cursors.push(Arc::clone(b));
+                cursors.push_back(b.clone());
               }
             },
             $Node::Data(data,rs) => {
-              rows.extend(data.iter().map(|pv| {
-                (pv.0.clone(),pv.1.clone())
-              }).collect::<Vec<_>>());
+              rows.extend(data.iter().cloned().collect::<Vec<_>>());
               refs.extend_from_slice(&rs);
             },
           }
         }
         (rows,refs)
       }
+      fn list_refs(&mut self) -> Vec<TreeRef<($(Coord<$T>),+)>> {
+        let mut cursors = VecDeque::new();
+        cursors.push_back(self.root.clone());
+        let mut refs = vec![];
+        while let Some(c) = cursors.pop_front() {
+          match c.as_ref() {
+            $Node::Branch(branch) => {
+              for (_bitfield,b) in branch.intersections.iter() {
+                cursors.push_back(b.clone());
+              }
+              for b in branch.nodes.iter() {
+                cursors.push_back(b.clone());
+              }
+            },
+            $Node::Data(_data,rs) => {
+              refs.extend_from_slice(&rs);
+            },
+          }
+        }
+        refs
+      }
+      fn query_local(
+        &mut self, bbox: &(($($T),+),($($T),+))
+      ) -> (Vec<(($(Coord<$T>),+),V)>,Vec<TreeRef<($(Coord<$T>),+)>>) {
+        let mut rows = vec![];
+        let mut refs = vec![];
+        let mut cursors = VecDeque::new();
+        cursors.push_back((0,self.root.clone()));
+
+        while let Some((level,c)) = cursors.pop_front() {
+          match c.as_ref() {
+            $Node::Branch(branch) => {
+              match level % $dim {
+                $($i => {
+                  let pivots = branch.pivots.$i.as_ref().unwrap();
+
+                  {
+                    let mut matching: u32 = 0;
+                    if &(bbox.0).$i <= pivots.first().unwrap() {
+                      matching |= (1<<0);
+                    }
+                    let ranges = pivots.iter().zip(pivots.iter().skip(1));
+                    for (i,(start,end)) in ranges.enumerate() {
+                      if intersect_iv(start, end, &(bbox.0).$i, &(bbox.1).$i) {
+                        matching |= (1<<i);
+                        matching |= (1<<(i+1));
+                      }
+                    }
+                    if &(bbox.1).$i >= pivots.last().unwrap() {
+                      matching |= (1<<(pivots.len()-1));
+                    }
+                    for (bitfield,b) in branch.intersections.iter() {
+                      if (matching & bitfield) > 0 {
+                        cursors.push_back((level+1,Arc::clone(b)));
+                      }
+                    }
+                  }
+
+                  {
+                    let xs = &branch.nodes;
+                    let ranges = pivots.iter().zip(pivots.iter().skip(1));
+                    if &(bbox.0).$i <= pivots.first().unwrap() {
+                      cursors.push_back((level+1,Arc::clone(xs.first().unwrap())));
+                    }
+                    for ((start,end),b) in ranges.zip(xs.iter().skip(1)) {
+                      if intersect_iv(start, end, &(bbox.0).$i, &(bbox.1).$i) {
+                        cursors.push_back((level+1,Arc::clone(b)));
+                      }
+                    }
+                    if &(bbox.1).$i >= pivots.last().unwrap() {
+                      cursors.push_back((level+1,Arc::clone(xs.last().unwrap())));
+                    }
+                  }
+                }),+
+                _ => panic!["unexpected level modulo dimension"]
+              }
+            },
+            $Node::Data(data,rs) => {
+              rows.extend(data.iter()
+                .filter(|pv| {
+                  true $(&& intersect_coord(&(pv.0).$i, &(bbox.0).$i, &(bbox.1).$i))+
+                })
+                .cloned()
+                .collect::<Vec<_>>()
+              );
+              refs.extend(rs.iter()
+                .filter(|r| {
+                  true $(&& intersect_coord(&r.bounds.$i, &(bbox.0).$i, &(bbox.1).$i))+
+                })
+                .cloned()
+                .collect::<Vec<TreeRef<($(Coord<$T>),+)>>>()
+              );
+            },
+          }
+        }
+        (rows,refs)
+      }
       fn query<S>(&mut self, trees: Arc<Mutex<TreeFile<S,Self,($(Coord<$T>),+),V>>>,
         bbox: &(($($T),+),($($T),+)), fields: Arc<SetupFields>,
         root_index: usize, root_id: TreeId,
@@ -587,9 +713,15 @@ type CreateTrees<T> = HashMap<TreeId,Arc<Mutex<T>>>;
 pub trait Tree<P,V>: Send+Sync+ToBytes+FromBytes+CountBytes+std::fmt::Debug+'static
 where P: Point, V: Value {
   fn empty() -> Self;
-  fn build<'a>(fields: Arc<SetupFields>, rows: &[(P,InsertValue<'a,P,V>)], next_tree: &mut TreeId)
-    -> (Option<TreeRef<P>>,CreateTrees<Self>) where Self: Sized;
+  fn build<'a>(
+    fields: Arc<SetupFields>,
+    rows: &[(P,InsertValue<'a,P,V>)],
+    next_tree: &mut TreeId,
+    is_rm: bool,
+  ) -> (Option<TreeRef<P>>,CreateTrees<Self>) where Self: Sized;
   fn list(&mut self) -> (Vec<(P,V)>,Vec<TreeRef<P>>);
+  fn list_refs(&mut self) -> Vec<TreeRef<P>>;
+  fn query_local(&mut self, bbox: &P::Bounds) -> (Vec<(P,V)>,Vec<TreeRef<P>>);
   fn query<S>(&mut self, trees: Arc<Mutex<TreeFile<S,Self,P,V>>>, bbox: &P::Bounds,
     fields: Arc<SetupFields>, root_index: usize, root_id: TreeId) -> QStream<P,V> where S: RA;
   async fn remove<S>(&mut self, ids: Arc<Mutex<HashMap<V::Id,P>>>)
@@ -624,18 +756,12 @@ where P: Point, V: Value, T: Tree<P,V>, S: RA {
 
     for _ in 0..self.rebuild_depth {
       let mut trees = self.trees.lock().await;
-      let bounds = l_refs.iter().map(|r| r.bounds.clone()).collect::<Vec<P>>();
-      let intersecting = calc_overlap::<P>(&bounds);
       let mut n_refs = vec![];
-      for (r,overlap) in l_refs.iter().zip(intersecting) {
-        if overlap {
-          let (list,xrefs) = trees.get(&r.id).await?.lock().await.list();
-          lists.push(list);
-          n_refs.extend(xrefs);
-          rm_trees.push(r.id);
-        } else {
-          rows.push((r.bounds.clone(), InsertValue::Ref(r.clone())));
-        }
+      for r in l_refs.iter() {
+        let (list,xrefs) = trees.get(&r.id).await?.lock().await.list();
+        lists.push(list);
+        n_refs.extend(xrefs);
+        rm_trees.push(r.id);
       }
       l_refs = n_refs;
       if l_refs.is_empty() { break }
@@ -656,13 +782,14 @@ where P: Point, V: Value, T: Tree<P,V>, S: RA {
     let (tr, create_trees) = T::build(
       Arc::clone(&self.fields),
       &rows,
-      &mut self.next_tree
+      &mut self.next_tree,
+      false
     );
     Ok((tr, rm_trees, create_trees))
   }
   pub async fn remove(&mut self) -> Result<(),Error> {
     if self.deletes.is_empty() { return Ok(()) }
-    let mut join = Join::new();
+    let mut work = vec![];
     let ids = {
       let mut map = HashMap::new();
       for d in self.deletes.iter() {
@@ -684,7 +811,7 @@ where P: Point, V: Value, T: Tree<P,V>, S: RA {
       let xids = Arc::clone(&ids);
       let id = r.id;
       let xfields = Arc::clone(&fields);
-      join.push(async move {
+      work.push(async move {
         let mut refs = vec![id];
         while !refs.is_empty() {
           let r = refs.pop().unwrap();
@@ -709,7 +836,8 @@ where P: Point, V: Value, T: Tree<P,V>, S: RA {
               let (tr, create_trees) = T::build(
                 Arc::clone(&xfields),
                 &rows,
-                &mut next_tree
+                &mut next_tree,
+                true
               );
               let tr_id = tr.map(|x| x.id);
               assert![tr_id == Some(r),
@@ -723,10 +851,11 @@ where P: Point, V: Value, T: Tree<P,V>, S: RA {
             }
           }
         }
-        Ok(())
+        let r: Result<(),Error> = Ok(());
+        r
       });
     }
-    join.try_join().await?;
+    for r in join_all(work).await { r?; }
     if self.error_if_missing {
       let xids = ids.lock().await;
       if !xids.is_empty() {
@@ -739,19 +868,6 @@ where P: Point, V: Value, T: Tree<P,V>, S: RA {
   }
 }
 
-fn calc_overlap<X>(bounds: &[X]) -> Vec<bool> where X: Overlap {
-  let mut res = vec![false;bounds.len()];
-  for i in 0..bounds.len() {
-    for j in i+1..bounds.len() {
-      if bounds[i].overlap(&bounds[j]) {
-        res[i] = true;
-        res[j] = true;
-      }
-    }
-  }
-  res
-}
-
 fn find_separation<X>(amin: &X, amax: &X, bmin: &X, bmax: &X, is_min: bool) -> X where X: Scalar {
   if is_min && intersect_iv(amin, amax, bmin, bmax) {
     amin.clone()/2.into() + bmin.clone()/2.into()
